from . RxComputeContext import RxComputeContext
from abc import ABCMeta
from typing import Callable

# Super class for RxLocalSeq and RxLocalParallel
# Its run method (called from rx_exec) executes tasks sequentially.
# RxLocalParallel calls super().run(...) in case of single argument to prevent needlessly
# spawning processes.
class RxLocalComputeContext(RxComputeContext, metaclass=ABCMeta):

    def __init__(self, derived_name, derived_version):
        super().__init__(derived_name, derived_version)

    def execute_tasks(self, function: Callable, args_for_tasks: list, times_to_run: int, task_chunk_size: int,
                      callback: Callable = None, **kwargs) -> list:
        results = [function(**args_for_tasks[x]) for x in range(times_to_run)]
        if callback is not None:
            results = [callback(results[x]) for x in range(times_to_run)]
        return results
