#
# Copyright (c) Microsoft Corporation.  All rights reserved.
#

import abc

from .RxRemoteJob import RxRemoteJob
from .RxComputeContext import RxComputeContext
from ..datasource import RxDataSource


#################################################################################
#
# Remote compute context
#
class RxRemoteComputeContext(RxComputeContext):
    __metaclass_ = abc.ABCMeta

    def __init__(self, description: str, version: str, id: str = None, wait: bool = True):
        super().__init__(description, version, id)

        self._wait = wait

    @property
    def wait(self) -> bool:
        return self._wait

    @abc.abstractmethod
    def create_job(self, inData: RxDataSource = None) -> RxRemoteJob:
        pass

    @abc.abstractmethod
    def get_job(self) -> RxRemoteJob:
        pass

    @abc.abstractmethod
    def start_job(self, job: RxRemoteJob):
        pass

    @abc.abstractmethod
    def poll_for_job_completion(self, job: RxRemoteJob):
        pass

    @abc.abstractmethod
    def get_job_state(self, job: RxRemoteJob) -> str:
        pass

    @abc.abstractmethod
    def cancel_job(self, job: RxRemoteJob):
        pass

    @abc.abstractmethod
    def get_job_result(self, job: RxRemoteJob) -> list:
        pass

    @abc.abstractmethod
    def delete_job(self, job: RxRemoteJob):
        pass

    @abc.abstractmethod
    def supports_job(self, job:RxRemoteJob) -> bool:
        pass

    @abc.abstractmethod
    def get_job_output(self, job: RxRemoteJob) -> str:
        pass
