from os import path
from typing import Union
import glob
import zipfile
import os
import platform
import sys

from pathlib import Path
from pandas import DataFrame

from revoscalepy.datasource import RxDataSource
from revoscalepy.datasource import RxXdfData
from revoscalepy.datasource import RxFileSystem
from revoscalepy.datasource import RxNativeFileSystem



def create_data_file(input_data) -> Union[DataFrame, RxDataSource]:
    if input_data is None or isinstance(input_data, DataFrame):
        return input_data

    if isinstance(input_data, str) and (len(input_data) == 0 or input_data.isspace()):
        return None

    filename, extension = path.splittext(input_data)

    if extension == ".xdf" or "":
        return RxXdfData(input_data)
    else:
        return None

def rx_zip(inputdir : str, outputfile: str) -> str:
    inputdir = os.path.normpath(inputdir).strip(os.sep)
    with zipfile.ZipFile(outputfile, 'w') as outzipfile:
        substrIndex = len(inputdir)
        for file in [f for f in glob.glob(pathname = inputdir + "\\**", recursive = True) if os.path.isfile(f)]:
            outzipfile.write(file, file[substrIndex:])
        
    return outputfile
                
def rx_unzip(inputfile : str, outputdir: str) -> str:
    with zipfile.ZipFile(inputfile, 'r') as inputzipfile:
        inputzipfile.extractall(outputdir)

def file_exists(file: str, filesystem: RxFileSystem=RxNativeFileSystem()):
    if isinstance(filesystem, RxNativeFileSystem):
        file_path = Path(file)
        return file_path.exists()
    return False

# Get the executable directory
def get_exec_dir():
    return os.path.join(os.path.dirname(sys.executable))


# Finds the R installation directory based upon it being relative to the python install directory
def get_r_install_dir():
    r_install_dirs = ["R_SERVER", "R_SERVICES", "R"]

    # Look for an R installation
    for install_dir in r_install_dirs:
        r_dir = os.path.join(os.path.dirname(get_exec_dir()), install_dir)
        if os.path.isdir(r_dir):
            return r_dir
    return None

# Gets the home directory of the user
def get_home_directory() -> Union[str, None]:
    home = None
    if platform.system() == 'Windows':
        home = os.environ['userprofile']
    else:
        home = os.environ['HOME']

    if home is None or not os.access(home, os.W_OK):
        return None

    return home

# Get the local config path of the user
def get_local_config_path() -> Union[str, None]:
    home = get_home_directory()

    if home is None:
        return None

    config_path = None
    if platform.system() == 'Windows':
        config_path = os.path.join(home, "AppData", "Local", "Microsoft", "R")
    else:
        config_path = os.path.join(home, ".config", "microsoft-r")

    if not os.access(config_path, os.W_OK):
        return None

    return config_path

# Retrieves the version of python
def get_python_version() -> str:
    return "{}.{}.{}".format(sys.version_info.major, sys.version_info.minor, sys.version_info.micro)
