from functools import wraps

def on_exit_handler(func):
    @wraps(func)
    def wrapper(*args, **kwds):
        __exit_handlers__ = list()
        try:
            return func(*args,**kwds)
        finally:
            for handler in __exit_handlers__:
               handler.exec()
    return wrapper

# on_exit handler that is used to register a function along with a list of parameters
def on_exit(func, *args, **kwargs):
    import inspect

    handler = ExitHandler(func,*args, **kwargs)

    # Get the frame for the on_exit_handler function
    current_frame = inspect.currentframe().f_back.f_back
    __on_exit_handler(handler, current_frame)

# exit handler that can accept a specified frame
def __on_exit_handler(exit_handler, frame):
    if frame is None:
        return

    locals = frame.f_locals
    if '__exit_handlers__'  in locals:
        locals['__exit_handlers__'].append(exit_handler)


# Helper class containing the function and parameters to call the function
class ExitHandler:
    def __init__(self, func, *args, **kwargs):
        self.func = func
        self.args = args
        self.kwargs = kwargs

    def exec(self):
        self.func(*self.args, **self.kwargs)

    def __del__(self):
        self.exec()