import inspect
import datetime
import os
import sys

from enum import Enum
from . RxOptions import RxOptions


class RxTraceLevel(Enum):
    NONE = 0
    ERROR = 1
    WARN = 2
    INFO = 3
    DEBUG = 4
    FINE = 5
    FINE2 = 6
    TRACE = 7


def _rx_runtime_print(message: str, timestamp: bool = False):
    if timestamp:
        print(datetime.datetime.utcnow().strftime("%H:%M:%S.%f") + ", ", end="")

    print(str(os.getpid()) + ":", end="")

    stack = inspect.stack()
    indent_count = len(stack)
    for i in range(0, indent_count):
        print("    ", end="")

    fun_frame = stack[2]
    print("~~~~{0}::{1}({2}), {3}".format(os.path.basename(fun_frame[1]), fun_frame[3], fun_frame[2], message))
    sys.stdout.flush()


def rx_runtime_trace(message: str, trace_level: RxTraceLevel):
    user_trace_level = RxOptions.get_option("traceLevel", 0)
    if trace_level.value <= user_trace_level:
        _rx_runtime_print(message, RxOptions.get_option("timestamp-utc", False))


def init_runtime_trace():
    trace_mapping = {
        "NONE": 0,
        "ERROR": 1,
        "WARN": 2,
        "INFO": 3,
        "DEBUG": 4,
        "FINE": 5,
        "FINE2": 6,
        "TRACE": 7
    }

    if "MRS_TRACE_LEVEL" in os.environ:
        runtime_trace_level = str(os.environ["MRS_TRACE_LEVEL"])
        if runtime_trace_level in trace_mapping:
            # runtime tracing set trace level directly to not trigger native call
            # bxlserver will init runtime tracing from env by itself
            RxOptions.instance()["traceLevel"] = trace_mapping[runtime_trace_level]

    if "MRS_TRACE_EXTRAS" in os.environ:
        runtime_trace_extras = str(os.environ["MRS_TRACE_EXTRAS"])
        RxOptions.set_option("timestamp-utc", "timestamp-utc" in runtime_trace_extras)
